/******************************************************************************** 
 * File:   create.h
 * Author: skidmarks
 *
 * Created on March 3, 2009, 11:14 AM
********************************************************************************/

//********************************************************************************
// PURPOSE:
//    Base class for partition creation. Contains common create functions.
//
// DESCRIPTION:
//    The base class contains required statistics reporting functions and
//    a common  list creation function. The shell provides a framework for
//    instantiating the class.
//    
//
// ARCHITECTURE:
//    Class functions are:
//    
//    Function        Description
//    cCreate         Class Copy and Class Constructor
//    ~cCreate        Class destructor
//    Binary          Virtual function to open/close binary stream
//    BinaryOutput    Output binary represenation of partitions
//    Create_DataList List of references within a partition
//    Detail          Output partitions and their references
//    Histogram       Output all partitions and reference histogram
//    Statistics      Output general partition statistics
//    operator[]      Access a partitions element by index
//    
// NOTES:
//    1. Run(), the entry point for partition creation, is not included as
//       a virtual function because it has two forms. A qualitative and 
//       poorly conceived reason to exclude it.
//********************************************************************************

#ifndef _CREATE_H
#define	_CREATE_H

# include "stdafx.h"

# include <fstream>
# include <sstream>
# include <ctime>
# include "Debug.h"
# include "common.h"
# include "irange.h"

  class cCreate : public cDebug {
  protected:
     enum { MAXIMUM_ERROR_COUNT = 100 };
     cBound_List*     Bounds;                    // <bound, type, &Range>
     cPartition_List* Partitions;                // Partition array
     const double     Bandwidth;                 // Maximum - minimum values
     const char*      Filename;                  // Input data file name
     time_t           StartTime;                 // Begin time for algorithm
     const string     Name;                      // Name of partition
  protected:
    cCreate(const char* myFilename, const string myName, const double myBandwidth, cBound_List* myBounds) 
                                                       : Bandwidth(myBandwidth) 
                                                       , Bounds(myBounds) 
                                                       , Filename(myFilename)
                                                       , StartTime(clock())
                                                       , Name(myName) { };
    cList* Create_DataList(cStack& Stack);
    void   BinaryOutput(string suffix);
    virtual
    void DData () { ostringstream stream(ios::out); }
  public:
    cCreate(const cCreate& orig);
    virtual void Binary() = 0;

// REPORTS
    void Detail();
    void Histogram();
    void Statistics();

    virtual ~cCreate() { };
    cPartition&  operator[](long Ndx)  { return (*Partitions)[Ndx];  };
  }; // class cCreate

#endif	/* _CREATE_H */

