/* 
 * File:   partition.h
 * Author: Arthur Schwarz
 *
 * Created on April 17, 2009, 10:48 AM
 */

#ifndef _PARTITION_H
#define	_PARTITION_H

# include "stdafx.h"

# include <iomanip>
# include <fstream>

class cOption : public cDebug {                  // description of argument
private:
protected:
   virtual
   void DData () { ostringstream stream(ios::out);
                   stream << *this;
                   DebugData(stream.str());
   }
public:
   typedef long (*OptionFun)(cOption& Option);
   enum eFlag { eOptional    = 0                 // Optional option w/no arguments
              , eArgument    = 1                 // Argument allowed (=0 argument not allowed)
              , eArgument_Req= 2                 // Argument required (-x arg)
              , eRequired    = 4                 // Option required
              };
   char  const  ShortName;                       // [-]<letter>
   char* const  LongName;                        // [-]'-'<string>
   bool         Empty;                           // argument found (false)
   long   const Flag;                            // type of option/argument
   OptionFun    Function;                        // Option handling function
   char*        Val;                             // string representing argument value
   cOption() : ShortName('\0'), LongName('\0'), Empty(false), Flag(0), Val("") { };
   cOption(char myShort, char* myLong, long myFlag, OptionFun myFunction)
                                      : ShortName(myShort)
                                      , LongName(myLong)
                                      , Empty(true)
                                      , Flag(myFlag)
                                      , Function(myFunction)
                                      , Val(NULL) { }
   friend  ostream& operator<<(ostream& stream, cOption& Option);
}; // cOption

class cOption_List : public cDebug {
private:
    static  cOption ErrorOption;                 // Faulty return
    cOption**  Option;                           // List of options
    long const Size;                             // Number of items in the list
protected:
      virtual
      void DData () { ostringstream stream(ios::out);
                      stream << "Option List";
                      DebugData(stream.str());
                      for(long i = 0; i < Size; i++) {
                          stream.flush();
                          stream << "      " << Option[i];
                          DebugData(stream.str());
                      }
                    }
public:
    cOption_List(long const mySize, cOption* myOption) : Size(mySize) {
        Option = new cOption*[Size];
        for(long i = 0; i < Size; i++) Option[i] = &myOption[i];
    }
    ~cOption_List() { delete [] Option; }
    long     GetSize() { return Size; }
    cOption& operator[](long Ndx) { return ((Ndx < 0) || (Ndx >= Size))? ErrorOption: *Option[Ndx]; }
}; // cOption_List

    //----------------------------------------
    //            DEBUG STREAMS
    //----------------------------------------
ostream& operator<<(ostream& stream, cOption& Option) {
    static const string Flag[] = { "Optional Option, No Argument"
                                 , "Optional Option, Optional Argument"
                                 , "Optional Option, Required Argument"
                                 , "Optional Option, Required Argument"
                                 , "Required Option, No Argument"
                                 , "Required Option, Optional Argument"
                                 , "Required Option, Required Argument"
                                 , "Required Option, Required Argument"
                                 };
    string const Empty = (Option.Empty)? "Not Used": "Used";
    char*  const Val   = (Option.Val)? Option.Val: "No Value";
    stream << "<"  << Option.ShortName
           << ", " << Empty
           << ", " << Flag[(long)Option.Flag]
           << ", " << Val
           << ">";
    return stream;
}; // ostream& operator<<(ostream& stream, cOption& Option)


#endif	/* _PARTITION_H */

