/******************************************************************************** 
 * File:   utility.h
 * Author: A. Schwarz
 *
 * Created on May 15, 2009, 12:01 PM
********************************************************************************/

//********************************************************************************
// PURPOSE:
//    A set of classless functions serving non-specific purposes.
//
// DESCRIPTION:
//    A general set of functions supported overall processing which don't 
//    fit neatly into a single category. The functions and classes support
//    processing by more than one class but are class inspecific.
//    
// ARCHITECTURE:
//    sFiles    Structure of file names needed in processing
//    Basename  Return the file path w/o the file name
//    diffclock Subtract clock_t time and return the resultant in Seconds
//    is_Num;   Determines if the input is a number (integer or floating point)
//    Open      Open a file using the provided mode.
//    Parse     Parse an input number.
//    Strip     Strip comments from an input file.
//    
// NOTES:
//    
//********************************************************************************


#ifndef _UTILITY_H
#define	_UTILITY_H

# include <ctime>
# include <fstream>
# include <string>

using namespace std;

struct sFiles {
   string Basename;                              // Basename for files
   string Generate;                              // Generation input Statistics file
   string Interval;                              // Partition input file
   string Log;                                   // Output log file
   string Output;                                // Final output file
   sFiles() : Log(""), Generate(""), Interval(""), Output("") { }
};

//********************************************
//             Shared Data
//********************************************

extern sFiles       File;                       // Filenames

//********************************************
//          Utility Functions
//********************************************

string Basename(string Filename);
double diffclock(clock_t End, clock_t Start);
bool   is_Num(const string& Str, long& Start, long& Stop);
bool   Open(ifstream& stream, const string& Filename, ios_base::openmode Mode);
bool   Open(ofstream& stream, const string& Filename, ios_base::openmode Mode);
bool   Parse(const string& Str, long& Start, long& Stop); // Parse input line
string StripComments(const string& Line);                 // Parse input line

#endif
